# Imports
import graphviz
import importlib
import inspect
import pydclass.class_library as library
import os
import yaml

def render(format):
    try:
        file_path = os.path.realpath(__file__)

        dot = graphviz.Digraph('structs', filename='structs_revisited.gv',
                     node_attr={'shape': 'record'})

        template_node = """<<TABLE BORDER="0" CELLBORDER="0" CELLSPACING="0">
        <TR>
        <TD fontcolor=\"red\"><font color=\"red\" style=\"Arial\">{title}</font></TD></TR>
        {meth_list}
        {attr_list}
  
        </TABLE>>"""

        file_list = os.listdir('.')
        try:
            with open(os.getcwd()+'/pydclass-config.yaml') as file: 
                config = yaml.load(file, Loader=yaml.FullLoader)
        except FileNotFoundError:
            print('[!] No configuration file found; Using the default configuration.')
            config = {"attributs_color": "blue", "methods_color": "orange", 'text_color': 'white'}
        try:
            meth_template = "<TR><TD BGCOLOR='"+config['methods_color']+"'><font color='"+config['text_color']+"'>Methods: </font></TD><TD>   {meth}</TD></TR>"
            attr_template = "<TR><TD BGCOLOR='"+config['attributs_color']+"'><font color='"+config['text_color']+"'>Attributes: </font></TD><TD>   {attr}</TD></TR>"
        except KeyError:
            print('[!] The program cannot read the README.md file; There is some information missing. Automatic switch to default configuration.')
            config = {"attributs_color": "blue", "methods_color": "orange", 'text_color': 'white'}
            meth_template = "<TR><TD BGCOLOR='"+config['methods_color']+"'><font color='"+config['text_color']+"'>Methods: </font></TD><TD>   {meth}</TD></TR>"
            attr_template = "<TR><TD BGCOLOR='"+config['attributs_color']+"'><font color='"+config['text_color']+"'>Attributes: </font></TD><TD>   {attr}</TD></TR>"
        module_list = []
        for f in file_list:
            if f.split('.')[-1] == "py":
                module_list.append(f.split('.')[0])

        members = []

        for m in module_list:
            members += inspect.getmembers(importlib.import_module(m))

        class_list = []
        for m in members:
            if inspect.isclass(m[1]):
                class_list.append(m[1])

        attributes_h_list = []
        methods_h_list = []
        final_dict = []
        meth = []

        for c in class_list:
            ci = library.class_infos(c, importlib.import_module(module_list[0]), methods_h_list, attributes_h_list)
            meth_list = ci.class_methods()
            for m in meth_list:
                meth.append(m.__name__)
            attr_list = ci.class_atributes(meth)
            parents_list = ci.class_parents()

            dictionary = {"name": c.__name__, "methods": meth_list, "attributes": attr_list, "parents": parents_list}
            attributes_h_list += ci.get_attributes_list()
            methods_h_list += ci.get_methods_list()
            final_dict.append(dictionary)


        for c in final_dict:
            attrib = []
            meth = []
            parents_attr_list = []


            for m in c['methods']:
                
                meth.append(meth_template.format(meth=m.__name__))
            meth = "".join(meth)
            for a in c['attributes']:

                attrib.append(attr_template.format(attr=a))
            attrib = "".join(attrib)

            dot.node(c['name'], label=template_node.format(title=c['name'], meth_list=meth, attr_list=attrib))

        for c in final_dict:
            for p in c['parents']:
                dot.edge(p.__name__, c['name'], color="blue")



        dot.render(os.path.dirname(os.path.realpath(__file__))+"/graph", format=format).replace('\\', '/')
    except BaseException as err:
        dot.node('Error occured', label="<TR><TD fill='red'>An error occurred during the build.</TR></TD><TR><TD>You can make a request on GitHub clearly stating this: "+str(err)+".</TD></TR>")
        dot.render(os.path.dirname(os.path.realpath(__file__))+"/graph", format=format)
def get_svg_datas(format):
    render(format)
    render_datas = open(os.path.dirname(os.path.realpath(__file__))+'/graph.svg', 'r').read()
    return render_datas 


#
# -*/*- PyDClass Library -*/*-
# Get class informations
# Author: VitriSnake
# License: GPLv3.0
#
# You can use this program but it's devlopped FOR PyDClass so please don't report if bug is detected into an other program using this library.
# Follow us on GitHub: https://github.com/CeltiumDev/PyDClass

# Imports
import inspect, importlib
import sys
import ipdb # /!\ For debug, you can safety delete this line.


class class_infos:
    def __init__(self, class_obj, module_obj, methods_list = [], attributes_list = []):
        self.class_obj = class_obj
        self.module_obj = module_obj
        self.methods_list = methods_list # For don't have parents/child
        self.attributes_list = attributes_list # For don't have parents/child

    def class_methods(self, name=False):
        class_members = inspect.getmembers(self.class_obj)
        class_methods = []
        for member in class_members:
            if inspect.isfunction(member[1]): # Check if is a method
                if not(member[1] in self.methods_list) and not(member[0].startswith('__')): 
                    if name == False:
                        self.methods_list.append(member[1])
                        class_methods.append(member[1])
                    elif name == True:
                        self.methods_list.append(member[1])
                        class_methods.append(member[0])
                    else:
                        raise ValueError('No valable values. -> name = True or False not '+str(name))

        return class_methods

    def class_atributes(self, methods_list):
        class_members = inspect.getmembers(self.class_obj())
        class_attributes_list = []
        for member in class_members:
            if not(member[0] in methods_list):
                if not(member[0].startswith('__')) and not(member[0] in self.methods_list):
                    class_attributes_list.append(member[0])
                    self.attributes_list.append(member[0])

        return class_attributes_list

    def class_parents(self):
        parents_list = []
        for p in self.class_obj.__bases__:
            if not(p.__name__.startswith('__')):
                if p.__name__ == "object":
                    pass
                else: 
                    parents_list.append(p)

        return parents_list

    def get_attributes_list(self):
        return self.attributes_list

    def get_methods_list(self):
        return self.methods_list

if __name__ == "__main__":
    import test1
    """
    1) Récupérer les class du module
    2) Faire passer le programme tout en fusionant les méthodes et attributs récupérés
    3) Retourner sous forme de dictionaire
    """
    members = inspect.getmembers(test1)
    class_list = []
    for m in members:
        if inspect.isclass(m[1]):
            class_list.append(m[1])

    attributes_h_list = []
    methods_h_list = []
    final_dict = []
    meth = []

    print(class_list)
    print(inspcet.getmembers)

    for c in class_list:
        ci = class_infos(c, test1, methods_h_list, attributes_h_list)
        meth_list = ci.class_methods()
        attr_list = ci.class_atributes(meth)
        parents_list = ci.class_parents()

        print(meth_list)
        print(attr_list)
        print(parents_list)

        dictionary = {"name": c.__name__, "methods": meth_list, "attributes": attr_list, "parents": parents_list}
        attributes_h_list += ci.get_attributes_list()
        methods_h_list += ci.get_methods_list()
        final_dict.append(dictionary)

    print(final_dict)



INDEX = """<!DOCTYPE html>
<html>
    <head>
        <title>Welcome - PyDClass</title>
        <meta charset="UTF-8"/>
        <meta http-equiv="refresh" content="5">
        <script src="https://cdnjs.cloudflare.com/ajax/libs/socket.io/4.0.1/socket.io.js" integrity="sha512-q/dWJ3kcmjBLU4Qc47E4A9kTB4m3wuTY7vkFJDTZKjTs8jhyGQnaUrxa0Ytd0ssMZhbNua9hE+E7Qv1j+DyZwA==" crossorigin="anonymous"></script>

        <script>

// When the user scrolls the page, execute myFunction
window.onscroll = function() {myFunction()};

// Get the header
var header = document.getElementById("myHeader");

// Get the offset position of the navbar
var sticky = header.offsetTop;

// Add the sticky class to the header when you reach its scroll position. Remove "sticky" when you leave the scroll position
function myFunction() {
  if (window.pageYOffset > sticky) {
    header.classList.add("sticky");
  } else {
    header.classList.remove("sticky");
  }
} 
        </script>
        <style>
             /* Style the header */
.header {
  padding: 10px 16px;
  background: #555;
  color: #f1f1f1;
}

/* Page content */
.content {
  padding: 16px;
}

/* The sticky class is added to the header with JS when it reaches its scroll position */
.sticky {
  position: fixed;
  top: 0;
  width: 100%
}

/* Add some top padding to the page content to prevent sudden quick movement (as the header gets a new position at the top of the page (position:fixed and top:0) */
.sticky + .content {
  padding-top: 102px;
} 
.content {
    text-align: center;
}

a {
    font-size: 10pt;
    color: black;
    text-decoration: none;
    font-family: Arial, Helvetica, sans-serif;

}
body {
    font-family: Arial, Helvetica, sans-serif;

}
h2 {
    color: orange;

}
#module_info {
    color: white;
    font-size: 15px;
    margin-left: 20px;
}
TR {
    margin: 100px;
}
        </style>

        
    </head>
    <body>
        <div class="header" id="myHeader">
            <h2>PyDClass <span id="module_info">Module loaded: {{module}}</span></h2>
        </div>
        <div class="content">
        
        <div id="svg">
        {{svg}}
        </div>
        </div>
    </body>
</html>"""
CHANGEMODULE = """<html lang="en">
    <head>
        <title>Change module - PyDClass</title>
        <meta charset="utf-8"/>
        <style>
            body {
                text-align: center;
                font-family: Arial, Helvetica, sans-serif;
            }
            input {
                border: 0;
                padding: 5px;
            }

            form {
                border: 1px solid black;
                display: inline-block;
            }
            .Input {
                width: 100px;
            }
        </style>
    </head>
    <body>
        <h2>Change module</h2>
        <p style="color: red"><b>Warning! </b>Not supported. Why? Beacause when you editing the new file, you don't look the new file but the old file. </p>
        <form method="GET" action="/module/form">
            <input id="Input" type="text" name="module" value="Not supported - That isn't an error" width="100" required="required" title="Not supported. Please go in our GitHub. "disabled/>
            <input type="submit" title="You can't send. Not supported. "disabled></input>
        </form>
    </body>
</html>"""


from crypt import methods
import os
from flask import *
from flask_socketio import SocketIO
import argparse
import logging
import argparse
import pydclass.gen_img

logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.DEBUG)

parser = argparse.ArgumentParser()
parser.add_argument("module", help="display a diagram of the module")
args = parser.parse_args()

app = Flask(__name__)
app.config['SECRET_KEY'] = 'ba971ddc5eb3fe749804bf2095dc6237c53e93053720b5d6d96270b26fd2c1ff'
socketio = SocketIO(app)


with open(f'{app.static_folder}/diagram.svg', ) as f:
        img = f.read()

socketio.emit('my_response',
             {'data': 'Server generated event',
              'image': img})


socketio.emit('my_response',
             {'data': 'Server generated event', 'count': 1,
              'image': img})

@app.route('/d/class')
def return_main_page():
    
    os.system('python3 '+os.getcwd()+'/gen_img.py svg '+args.module+' static/diagram')
    return render_template('main.html', module=args.module)

@app.route('/module/change')
def change_module():
    return render_template('changemodule.html', module=args.module)

@app.route('/module/form', methods=['GET'])
def get_method():
    module = request.args.get('module')
    args.module = module
    print(module)
    os.system('python3 '+os.getcwd()+'/gen_img.py svg '+args.module+' static/diagram')
    return redirect('/')

@app.route('/')
def index():
    return render_template('index.html', module=args.module)

if __name__ == "__main__":
    socketio.run(app, port=8080, host="0.0.0.0")
    


import random
import re
import sys
from flask import Flask, render_template

app = Flask(__name__)

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/page2')
def page2():
    return render_template('page2.html')

@app.context_processor
def inject_load():
    if sys.platform.startswith('linux'): 
        with open('/proc/loadavg', 'rt') as f:
            load = f.read().split()[0:3]
    else:
        load = [int(random.random() * 100) / 100 for _ in range(3)]
    return {'load1': load[0], 'load5': load[1], 'load15': load[2]}

app.run()


from setuptools import setup, find_packages
import shutil

setup(name = 'PyDClass',
      version = 2.0,
      author = 'VitriSnake',
      author_email = 'vitrisnake@celtiumc.fr',
      maintainer = 'CeltiumC',
      maintainer_email = 'contact@celtiumc.fr',
      description = 'PyDClass is a simple web class diagram generator',
      license = 'GPLv3.0',
      packages = find_packages(),
      package_data={'pydclass.templates': ['*'], 'pydclass.static': ['*']},
      include_package_data=True, 
      scripts=[
          'bin/pydclass',
      ],
      zip_safe=False,
      url='https://github.com/CeltiumCDev/PyDClass',
      keywords='diagram class project python generator web svg directory module',
      install_requires=[
          'flask',
          'graphviz',
          'ipdb'
      ],
      long_description=open('README.md', 'r').read(),
      long_description_content_type='text/markdown'
     )
